/* 
 * load/store config file
 *  d - system dependent
 */

#ifndef WIN32
	#define _GNU_SOURCE
	#include <stdio.h>
	#include <string.h>
	#include <stdlib.h>
#else
	#error "FALTEN INCLUDES PER A WINDOWS!!!"
        #error "FALTA MODIFICAR CODI PER A  WINDOWS!!!"
#endif
#include "d_workConfig.h"

#define BUFSIZE 100



// Does:
//  loads the config file 'file' consistent of 'slots' pairs "key value" (one
//  per line) on an undefined order and possibly with empty lines ("\n"), and 
//  scans for the keys defined on 'keys' and stores values on the array of 
//  char* 'values' (on the same relative position as keys)
//
// Returns:
//  - OK
//  - EFILE: could not open file
//  - ESTRUC: file format not correct
int loadConfig (const char *file, const int slots, const char **keys, char **values) {
    FILE *fd;
    char *buf, *token;
    int buflen, vpos;
    int tlen;
    int slot=0;

    if ((fd=fopen(file,"r")) == NULL)
	return EFILE;

    while (!feof(fd) && slot<slots) {
	buf=NULL;
	buflen=0;
	getline(&buf,(size_t*)&buflen,fd);
	if ((token=strtok(buf," \n"))!=NULL) {
	    if ((vpos=searchKeyPos(token,keys,slots))<0) {
		fclose(fd);
		return ESTRUC;
	    }
	    if ((token=strtok(NULL," \n"))==NULL) {
		fclose(fd);
		return ESTRUC;
	    }
	    tlen=strlen(token)+1;
	    values[vpos]=new char[tlen];
	    memcpy(values[vpos],token,tlen);
	    if (strtok(NULL," ")!=NULL) {
		fclose(fd);
		return ESTRUC;
	    }
	    slot++;
	}
	free(buf);
    }
    fclose(fd);
    if (slot<slots)
	return ESTRUC;
    return OK;
}

// Does:
//  scans for the position of 'key' on 'keys' (which has 'klen' elements)
//
// Returns:
//  - -1: 'key' is not on 'keys'
//  - otherwise: 'key' position
int searchKeyPos (const char *key, const char **keys, const int klen) {
    int i;

    for (i=0; i<klen; i++) {
	if (strcmp(key,keys[i])==0)
	    return i;
    }
    return -1;
}

// Does:
//  saves the config file 'file' with 'slots' pairs "key value" (one per
//  line), where all keys are on 'keys' and all values on 'values' (they are
//  stored in the same order, that is "keys[i] values[i]", and consecutive)
//
// Returns:
//  - OK
//  - EFILE: could not open file
int saveConfig (const char *file, const int slots, const char **keys, const char **values) {
    FILE *fd;

    if ((fd=fopen(file,"w")) == NULL)
	return EFILE;
    for(int i=0; i<slots; i++) {
	if(fprintf(fd,"%s %s\n",keys[i],values[i]) != strlen(keys[i])+strlen(values[i])+2) {
	    fclose(fd);
	    return EFILE;
	}
    }
    fclose(fd);
    return OK;
}

